#define KBUILD_MODNAME "blub"

#include <linux/bpf.h>
#include <linux/if_ether.h>
#include <linux/if_link.h>
#include <linux/if_xdp.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/udp.h>

#include "bpf_helpers.h"
#include "bpf_endian.h"
#include "parsing_helpers.h"

#define UDP_PORT 50000

#define bpf_printk(fmt, ...)                                               \
	({                                                                 \
		char ____fmt[] = fmt;                                      \
		bpf_trace_printk(____fmt, sizeof(____fmt), ##__VA_ARGS__); \
	})

struct bpf_map_def SEC("maps") xsks_map = {
	.type = BPF_MAP_TYPE_XSKMAP,
	.key_size = sizeof(int),
	.value_size = sizeof(int),
	.max_entries = 64,
};

SEC("xdp_sock")
int xdp_sock_prog(struct xdp_md *ctx)
{
	int eth_type, ip_type, index;
	struct ethhdr *eth;
	struct iphdr *iphdr;
	struct ipv6hdr *ipv6hdr;
	void *data_end = (void *)(long)ctx->data_end;
	void *data = (void *)(long)ctx->data;
	struct hdr_cursor nh = { .pos = data };

        index = ctx->rx_queue_index;

	eth_type = parse_ethhdr(&nh, data_end, &eth);
	if (eth_type < 0)
		return XDP_PASS;

	if (eth_type == bpf_htons(ETH_P_IP)) {
		ip_type = parse_iphdr(&nh, data_end, &iphdr);
	} else if (eth_type == bpf_htons(ETH_P_IPV6)) {
		ip_type = parse_ip6hdr(&nh, data_end, &ipv6hdr);
	} else {
		return XDP_PASS;
	}

	// only support UDP for now
	if (ip_type != IPPROTO_UDP)
		return XDP_PASS;

	struct udphdr *udphdr;
	// don't mess with ports outside our purview, if specified
	if (parse_udphdr(&nh, data_end, &udphdr) < 0)
		return XDP_PASS;
	if (bpf_ntohs(udphdr->dest) != UDP_PORT)
		return XDP_PASS;

	/* If socket bound to rx_queue then redirect to user space */
	if (bpf_map_lookup_elem(&xsks_map, &index))
		return bpf_redirect_map(&xsks_map, index, 0);

	/* Else pass to Linux' network stack */
	return XDP_PASS;
}

char _license[] SEC("license") = "GPL";

