#ifndef UTILITIES_H
#define UTILITIES_H

#define _GNU_SOURCE
#include <inttypes.h>
#include <signal.h>
#include <stdint.h>
#include <time.h>
#include <unistd.h>

#ifdef WITH_XDP
#include <bpf/bpf.h>
#include <bpf/libbpf.h>
#include <bpf/xsk.h>
#include <linux/if_ether.h>
#include <linux/if_link.h>
#include <linux/if_xdp.h>
#include <linux/ip.h>
#include <linux/udp.h>
#endif

#define NSEC_PER_SEC UINT64_C(1000000000)
#define SERVER_PORT "50000"
#define SERVER_PORT_INT 50000

#define MAX_KERNEL_LATENCY 1000
#define MAX_RTT 1000
#define MAX_JITTER 1000

#define MAX_BUFFER_SIZE 1024

#define TIMESTAMP_BUFFER_SIZE 4096
#define err(...)                              \
	do {                                  \
		fprintf(stderr, __VA_ARGS__); \
		exit(EXIT_FAILURE);           \
	} while (0)

#define err_errno(...) error(EXIT_FAILURE, errno, __VA_ARGS__);

uint64_t ts_to_uint(struct timespec t);
struct timespec uint_to_ts(uint64_t t);
void add_ns(struct timespec *t, uint64_t ns);
uint64_t calcdiff_ns(struct timespec t1, struct timespec t2);

void set_latency_target(void);
void init_signals(void (*_sighand)(int));

int64_t _min_(int64_t a, int64_t b);
int64_t _max_(int64_t a, int64_t b);

int histogram_min(uint64_t *histogram, int max_value);
int histogram_max(uint64_t *histogram, int max_value);

extern void (*previous_handlers[NSIG])(int);

void encode(uint64_t t, char * b);
uint64_t decode(char * b);

#endif
