#ifndef RECV_PACKET
#define RECV_PACKET

#include "common.h"

typedef struct ingress_param {
	char network_if[16];

	int use_timestamps;
	int xdp_polling_mode;
	uint64_t interval;

	size_t tx_buffer_len;

} ingress_param_t;

typedef struct ingress_stat {
	int min_kernel_latency;
	int avg_kernel_latency;
	int max_kernel_latency;

	int min_interval;
	int avg_interval;
	int max_interval;

	uint64_t packets_received;
	uint64_t high_kernel_latency;
	uint64_t high_jitter;

	char data[MAX_BUFFER_SIZE];
} ingress_stat_t;

void init_udp_recv(ingress_param_t *_params, ingress_stat_t *stats,
		   int use_histogram, uint64_t *_kernel_latency_hist);

void recv_udp_packet(void);

#ifdef WITH_XDP
#define NUM_FRAMES 4096
#define FRAME_SIZE XSK_UMEM__DEFAULT_FRAME_SIZE

struct xsk_umem_info {
	struct xsk_ring_prod fq;
	struct xsk_ring_cons cq;
	struct xsk_umem *umem;
	void *buffer;
};
struct xdpsock {
	struct xsk_ring_cons rx;
	struct xsk_ring_prod tx;
	struct xsk_umem_info umem;
	struct xsk_socket *xsk;
	int fd;
};
#endif

void init_xdp_recv(ingress_param_t * _params);
int recv_xdp_packet(void);
void recv_xdp_cleanup(void);
void setup_poll_fd(void);
void close_xdp_socket(void);

#endif
